// -----------------------------------------
// Schalten von Elro Funksteckdosen
// ATTINY 2313
// -----------------------------------------

#define F_CPU 1000000 // 1 MHz
#define funkpin PB1
#define ledpin PA1
#define HIGH 1
#define LOW 0


#include <avr/io.h>
#include <util/delay.h>
#include <stdbool.h>
#include <string.h>



//Wiederholungen der Funkbertragung
int repeat = 8;
char switchcon[12]="0F000FF0FF0F";
char switchcoff[12]="0F000FF0FFF0";


void SetupFunkPin(){
	// FunkPin als Ausgang schalten
	DDRB = (1 << funkpin);
}

void setpin(int pin, int state){
	if (state==HIGH){
		PORTB |= (1 << pin);
	}
	if (state==LOW){
		PORTB &= ~(1 << pin);
	}
}

void delayoneclk(int clks){
	for(int i=0; i<=clks; i++){
		_delay_us(342);
	}
}

void codeSenden(char *c){
	char ch;
	
	for(int i=0; i<=repeat; i++){
		for(int j=0; j<=11; j++){
			ch = c[j];
			if (ch=='0'){
				setpin(funkpin, HIGH);
				delayoneclk(1);
				setpin(funkpin, LOW);
				delayoneclk(3);
				setpin(funkpin, HIGH);
				delayoneclk(1);
				setpin(funkpin, LOW);
				delayoneclk(3);
			}
			if (ch=='F'){
				setpin(funkpin, HIGH);
				delayoneclk(1);
				setpin(funkpin, LOW);
				delayoneclk(3);
				setpin(funkpin, HIGH);
				delayoneclk(3);
				setpin(funkpin, LOW);
				delayoneclk(1);
			}
			if (ch=='1'){
				setpin(funkpin, HIGH);
				delayoneclk(3);
				setpin(funkpin, LOW);
				delayoneclk(1);
				setpin(funkpin, HIGH);
				delayoneclk(3);
				setpin(funkpin, LOW);
				delayoneclk(1);
			}
			
		}
		//Sende Sync 32 Takte
		setpin(funkpin, HIGH);
		delayoneclk(1);
		setpin(funkpin, LOW);
		delayoneclk(31);
		
	}
}

int main (void)
{
	
	PORTB = _BV(PB0);
	//Pin PB0 als Eingang definieren
	DDRB|=(1<<PB0);
	
	// Pin 1 von Port A (PA1) als Ausgang schalten
	DDRA = (1 << PA1);
	
	bool buttonpressed=false;
	bool ison=false;
	bool buttonreleased=true;
	SetupFunkPin();
	
	
	// Endlosschleife
	for (;;) {
		//Prfen, ob der Knopf gedrckt wurde
		if (bit_is_clear(PINB,0) && buttonreleased){
			buttonreleased=false;
			buttonpressed=true;
		}
		if (!bit_is_clear(PINB,0)){
			buttonreleased=true;
		}
		//Wenn der Knopf gedrckt und wieder losgelassen wurde...
		if (buttonpressed && buttonreleased){
			buttonpressed=false;
			//Wenn der bisherige Zustand nicht "on" war, einschalten
			if (!ison){
				PORTA |= (1 << ledpin);  // bit im Portregister auf 1 setzen => LED leuchtet
				codeSenden(switchcon);
				ison=true;
			}
			//Ansonsten ausschalten
			else{
				PORTA &= ~(1 << ledpin); // bit im Portregister auf 0 setzen => LED aus
				codeSenden(switchcoff);
				ison=false;
			}
		}
	}
}
